=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2008-03-26 09:50:48 +0000
+++ bzrlib/bzrdir.py	2008-03-28 08:05:51 +0000
@@ -2415,15 +2415,14 @@
     def initialize_on_transport(self, transport):
         try:
             # hand off the request to the smart server
-            shared_medium = transport.get_shared_medium()
+            client_medium = transport.get_smart_medium()
         except errors.NoSmartMedium:
             # TODO: lookup the local format from a server hint.
             local_dir_format = BzrDirMetaFormat1()
             return local_dir_format.initialize_on_transport(transport)
-        client = _SmartClient(shared_medium)
+        client = _SmartClient(client_medium, transport.base)
         path = client.remote_path_from_transport(transport)
-        response = _SmartClient(shared_medium).call('BzrDirFormat.initialize',
-                                                    path)
+        response = client.call('BzrDirFormat.initialize', path)
         assert response[0] in ('ok', ), 'unexpected response code %s' % (response,)
         return remote.RemoteBzrDir(transport)
 

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-03-27 00:27:41 +0000
+++ bzrlib/remote.py	2008-03-28 08:05:51 +0000
@@ -62,11 +62,11 @@
         self._real_bzrdir = None
 
         if _client is None:
-            self._shared_medium = transport.get_shared_medium()
-            self._client = client._SmartClient(self._shared_medium)
+            shared_connection = transport.get_shared_medium()
+            self._client = client._SmartClient(
+                shared_connection.connection, shared_connection.base)
         else:
             self._client = _client
-            self._shared_medium = None
             return
 
         path = self._path_for_remote_call(self._client)
@@ -277,7 +277,7 @@
             self._real_repository = None
         self.bzrdir = remote_bzrdir
         if _client is None:
-            self._client = client._SmartClient(self.bzrdir._shared_medium)
+            self._client = remote_bzrdir._client
         else:
             self._client = _client
         self._format = format
@@ -836,7 +836,7 @@
 
     def _get_parent_map(self, keys):
         """Helper for get_parent_map that performs the RPC."""
-        medium = self._client.get_smart_medium()
+        medium = self._client._medium
         if not medium._remote_is_at_least_1_2:
             # We already found out that the server can't understand
             # Repository.get_parent_map requests, so just fetch the whole
@@ -1055,7 +1055,7 @@
         return self._real_repository.has_signature_for_revision_id(revision_id)
 
     def get_data_stream_for_search(self, search):
-        medium = self._client.get_smart_medium()
+        medium = self._client._medium
         if not medium._remote_is_at_least_1_2:
             self._ensure_real()
             return self._real_repository.get_data_stream_for_search(search)
@@ -1229,7 +1229,7 @@
         if _client is not None:
             self._client = _client
         else:
-            self._client = client._SmartClient(self.bzrdir._shared_medium)
+            self._client = remote_bzrdir._client
         self.repository = remote_repository
         if real_branch is not None:
             self._real_branch = real_branch

=== modified file 'bzrlib/smart/client.py'
--- bzrlib/smart/client.py	2008-03-05 15:48:27 +0000
+++ bzrlib/smart/client.py	2008-03-28 08:05:51 +0000
@@ -23,15 +23,14 @@
 
 class _SmartClient(object):
 
-    def __init__(self, shared_connection):
+    def __init__(self, medium, base):
         """Constructor.
 
-        :param shared_connection: a bzrlib.transport._SharedConnection
+        :param medium: a SmartClientMedium
+        :param base: a URL
         """
-        self._shared_connection = shared_connection
-
-    def get_smart_medium(self):
-        return self._shared_connection.connection
+        self._medium = medium
+        self._base = base
 
     def call(self, method, *args):
         """Call a method on the remote server."""
@@ -47,7 +46,7 @@
             result, smart_protocol = smart_client.call_expecting_body(...)
             body = smart_protocol.read_body_bytes()
         """
-        request = self.get_smart_medium().get_request()
+        request = self._medium.get_request()
         smart_protocol = protocol.SmartClientRequestProtocolTwo(request)
         smart_protocol.call(method, *args)
         return smart_protocol.read_response_tuple(expect_body=True), smart_protocol
@@ -61,7 +60,7 @@
                 raise TypeError('args must be byte strings, not %r' % (args,))
         if type(body) is not str:
             raise TypeError('body must be byte string, not %r' % (body,))
-        request = self.get_smart_medium().get_request()
+        request = self._medium.get_request()
         smart_protocol = protocol.SmartClientRequestProtocolOne(request)
         smart_protocol.call_with_body_bytes((method, ) + args, body)
         return smart_protocol.read_response_tuple()
@@ -75,7 +74,7 @@
                 raise TypeError('args must be byte strings, not %r' % (args,))
         if type(body) is not str:
             raise TypeError('body must be byte string, not %r' % (body,))
-        request = self.get_smart_medium().get_request()
+        request = self._medium.get_request()
         smart_protocol = protocol.SmartClientRequestProtocolTwo(request)
         smart_protocol.call_with_body_bytes((method, ) + args, body)
         return smart_protocol.read_response_tuple(expect_body=True), smart_protocol
@@ -87,11 +86,11 @@
         anything but path, so it is only safe to use it in requests sent over
         the medium from the matching transport.
         """
-        base = self._shared_connection.base
+        base = self._base
         if base.startswith('bzr+http://') or base.startswith('bzr+https://'):
-            medium_base = self._shared_connection.base
+            medium_base = self._base
         else:
-            medium_base = urlutils.join(self._shared_connection.base, '/')
+            medium_base = urlutils.join(self._base, '/')
             
         rel_url = urlutils.relative_url(medium_base, transport.base)
         return urllib.unquote(rel_url)

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2008-03-28 06:42:20 +0000
+++ bzrlib/tests/test_remote.py	2008-03-28 08:05:51 +0000
@@ -140,7 +140,7 @@
         self.responses = responses
         self._calls = []
         self.expecting_body = False
-        _SmartClient.__init__(self, FakeMedium(fake_medium_base, self._calls))
+        _SmartClient.__init__(self, FakeMedium(self._calls), fake_medium_base)
 
     def call(self, method, *args):
         self._calls.append(('call', method, args))
@@ -162,14 +162,6 @@
 
 class FakeMedium(object):
 
-    def __init__(self, base, client_calls):
-        self.base = base
-        self.connection = FakeConnection(client_calls)
-        self._client_calls = client_calls
-
-
-class FakeConnection(object):
-
     def __init__(self, client_calls):
         self._remote_is_at_least_1_2 = True
         self._client_calls = client_calls

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2008-01-22 06:56:59 +0000
+++ bzrlib/tests/test_smart_transport.py	2008-03-28 08:05:51 +0000
@@ -2047,7 +2047,7 @@
         input = StringIO("\n")
         output = StringIO()
         client_medium = medium.SmartSimplePipesClientMedium(input, output)
-        smart_client = client._SmartClient(client_medium)
+        smart_client = client._SmartClient(client_medium, 'ignored base')
         self.assertRaises(TypeError,
             smart_client.call_with_body_bytes, method, args, body)
         self.assertEqual("", output.getvalue())

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2008-01-15 11:29:22 +0000
+++ bzrlib/transport/remote.py	2008-03-28 08:05:51 +0000
@@ -104,9 +104,12 @@
             self._shared_connection = transport._SharedConnection(medium,
                                                                   credentials,
                                                                   self.base)
+        else:
+            if medium is None:
+                medium = self._shared_connection.connection
 
         if _client is None:
-            self._client = client._SmartClient(self.get_shared_medium())
+            self._client = client._SmartClient(medium, self.base)
         else:
             self._client = _client
 

